import 'package:flutter/material.dart';

class ApplyColors {
  static ApplyColors instance = ApplyColors._(_generateColors());
  static final customColors = <String, Color>{};

  final Map<String, Map<int, String>> colorsAsMap;

  ApplyColors._(this.colorsAsMap);

  static Color getColor(String key) {
    String name;
    int shade = 500;

    var components = key.split("-");

    name = components.first;
    if (components.length > 1) {
      try {
        shade = int.parse(components[1]);
      } catch (e) {
        print(e);
      }
    }

    var customColor = customColors[name];

    if (customColor != null) {
      return createMaterialColor(customColor)[shade]!;
    }

    Map<int, String>? _fromHex = instance.colorsAsMap[name];
    if (_fromHex == null) {
      Color? materialColor = _SOURCE[name];

      if (materialColor == null) {
        throw "Nema boje";
      }
      try {
        return createMaterialColor(materialColor)[shade]!;
      } catch (e) {
        return Colors.black;
      }
    } else {
      return fromHex(_fromHex[shade]!);
    }
  }

  static Color fromHex(String hexString) {
    final buffer = StringBuffer();
    if (hexString.length == 6 || hexString.length == 7) buffer.write('ff');
    buffer.write(hexString.replaceFirst('#', ''));
    return Color(int.parse(buffer.toString(), radix: 16));
  }

  static MaterialColor createMaterialColor(Color color) {
    List strengths = <double>[.05];
    var swatch = <int, Color>{};
    final int r = color.red, g = color.green, b = color.blue;

    for (int i = 1; i < 10; i++) {
      strengths.add(0.1 * i);
    }
    strengths.forEach((strength) {
      final double ds = 0.5 - strength;
      swatch[(strength * 1000).round()] = Color.fromRGBO(
        r + ((ds < 0 ? r : (255 - r)) * ds).round(),
        g + ((ds < 0 ? g : (255 - g)) * ds).round(),
        b + ((ds < 0 ? b : (255 - b)) * ds).round(),
        1,
      );
    });
    return MaterialColor(color.value, swatch);
  }

}

Map<String, Map<int, String>> _generateColors() {
  Map<String, Map<int, String>> colorsAsMap = {};
  colorsAsMap['rose'] = {
    50: '#fffff1f2',
    100: '#ffffe4e6',
    200: '#fffecdd3',
    300: '#fffda4af',
    400: '#fffb7185',
    500: '#fff43f5e',
    600: '#ffe11d48',
    700: '#ffbe123c',
    800: '#ff9f1239',
    900: '#ff881337',
  };
  colorsAsMap['pink'] = {
    50: '#fffdf2f8',
    100: '#fffce7f3',
    200: '#fffbcfe8',
    300: '#fff9a8d4',
    400: '#fff472b6',
    500: '#ffec4899',
    600: '#ffdb2777',
    700: '#ffbe185d',
    800: '#ff9d174d',
    900: '#ff831843',
  };
  colorsAsMap['fuchsia'] = {
    50: '#fffdf4ff',
    100: '#fffae8ff',
    200: '#fff5d0fe',
    300: '#fff0abfc',
    400: '#ffe879f9',
    500: '#ffd946ef',
    600: '#ffc026d3',
    700: '#ffa21caf',
    800: '#ff86198f',
    900: '#ff701a75',
  };
  colorsAsMap['purple'] = {
    50: '#fffaf5ff',
    100: '#fff3e8ff',
    200: '#ffe9d5ff',
    300: '#ffd8b4fe',
    400: '#ffc084fc',
    500: '#ffa855f7',
    600: '#ff9333ea',
    700: '#ff7e22ce',
    800: '#ff6b21a8',
    900: '#ff581c87',
  };
  colorsAsMap['violet'] = {
    50: '#fff5f3ff',
    100: '#ffede9fe',
    200: '#ffddd6fe',
    300: '#ffc4b5fd',
    400: '#ffa78bfa',
    500: '#ff8b5cf6',
    600: '#ff7c3aed',
    700: '#ff6d28d9',
    800: '#ff5b21b6',
    900: '#ff4c1d95',
  };
  colorsAsMap['indigo'] = {
    50: '#ffeef2ff',
    100: '#ffe0e7ff',
    200: '#ffc7d2fe',
    300: '#ffa5b4fc',
    400: '#ff818cf8',
    500: '#ff6366f1',
    600: '#ff4f46e5',
    700: '#ff4338ca',
    800: '#ff3730a3',
    900: '#ff312e81',
  };
  colorsAsMap['blue'] = {
    50: '#ffeff6ff',
    100: '#ffdbeafe',
    200: '#ffbfdbfe',
    300: '#ff93c5fd',
    400: '#ff60a5fa',
    500: '#ff3b82f6',
    600: '#ff2563eb',
    700: '#ff1d4ed8',
    800: '#ff1e40af',
    900: '#ff1e3a8a',
  };
  colorsAsMap['lightBlue'] = {
    50: '#fff0f9ff',
    100: '#ffe0f2fe',
    200: '#ffbae6fd',
    300: '#ff7dd3fc',
    400: '#ff38bdf8',
    500: '#ff0ea5e9',
    600: '#ff0284c7',
    700: '#ff0369a1',
    800: '#ff075985',
    900: '#ff0c4a6e',
  };
  colorsAsMap['cyan'] = {
    50: '#ffecfeff',
    100: '#ffcffafe',
    200: '#ffa5f3fc',
    300: '#ff67e8f9',
    400: '#ff22d3ee',
    500: '#ff06b6d4',
    600: '#ff0891b2',
    700: '#ff0e7490',
    800: '#ff155e75',
    900: '#ff164e63',
  };
  colorsAsMap['teal'] = {
    50: '#fff0fdfa',
    100: '#ffccfbf1',
    200: '#ff99f6e4',
    300: '#ff5eead4',
    400: '#ff2dd4bf',
    500: '#ff14b8a6',
    600: '#ff0d9488',
    700: '#ff0f766e',
    800: '#ff115e59',
    900: '#ff134e4a',
  };
  colorsAsMap['emerald'] = {
    50: '#ffecfdf5',
    100: '#ffd1fae5',
    200: '#ffa7f3d0',
    300: '#ff6ee7b7',
    400: '#ff34d399',
    500: '#ff10b981',
    600: '#ff059669',
    700: '#ff047857',
    800: '#ff065f46',
    900: '#ff064e3b',
  };
  colorsAsMap['green'] = {
    50: '#fff0fdf4',
    100: '#ffdcfce7',
    200: '#ffbbf7d0',
    300: '#ff86efac',
    400: '#ff4ade80',
    500: '#ff22c55e',
    600: '#ff16a34a',
    700: '#ff15803d',
    800: '#ff166534',
    900: '#ff14532d',
  };
  colorsAsMap['lime'] = {
    50: '#fff7fee7',
    100: '#ffecfccb',
    200: '#ffd9f99d',
    300: '#ffbef264',
    400: '#ffa3e635',
    500: '#ff84cc16',
    600: '#ff65a30d',
    700: '#ff4d7c0f',
    800: '#ff3f6212',
    900: '#ff365314',
  };
  colorsAsMap['yellow'] = {
    50: '#fffefce8',
    100: '#fffef9c3',
    200: '#fffef08a',
    300: '#fffde047',
    400: '#fffacc15',
    500: '#ffeab308',
    600: '#ffca8a04',
    700: '#ffa16207',
    800: '#ff854d0e',
    900: '#ff713f12',
  };
  colorsAsMap['amber'] = {
    50: '#fffffbeb',
    100: '#fffef3c7',
    200: '#fffde68a',
    300: '#fffcd34d',
    400: '#fffbbf24',
    500: '#fff59e0b',
    600: '#ffd97706',
    700: '#ffb45309',
    800: '#ff92400e',
    900: '#ff78350f',
  };
  colorsAsMap['orange'] = {
    50: '#fffff7ed',
    100: '#ffffedd5',
    200: '#fffed7aa',
    300: '#fffdba74',
    400: '#fffb923c',
    500: '#fff97316',
    600: '#ffea580c',
    700: '#ffc2410c',
    800: '#ff9a3412',
    900: '#ff7c2d12',
  };
  colorsAsMap['red'] = {
    50: '#fffef2f2',
    100: '#fffee2e2',
    200: '#fffecaca',
    300: '#fffca5a5',
    400: '#fff87171',
    500: '#ffef4444',
    600: '#ffdc2626',
    700: '#ffb91c1c',
    800: '#ff991b1b',
    900: '#ff7f1d1d',
  };
  colorsAsMap['warmGray'] = {
    50: '#fffafaf9',
    100: '#fff5f5f4',
    200: '#ffe7e5e4',
    300: '#ffd6d3d1',
    400: '#ffa8a29e',
    500: '#ff78716c',
    600: '#ff57534e',
    700: '#ff44403c',
    800: '#ff292524',
    900: '#ff1c1917',
  };
  colorsAsMap['trueGray'] = {
    50: '#fffafafa',
    100: '#fff5f5f5',
    200: '#ffe5e5e5',
    300: '#ffd4d4d4',
    400: '#ffa3a3a3',
    500: '#ff737373',
    600: '#ff525252',
    700: '#ff404040',
    800: '#ff262626',
    900: '#ff171717',
  };
  colorsAsMap['gray'] = {
    50: '#fffafafa',
    100: '#fff4f4f5',
    200: '#ffe4e4e7',
    300: '#ffd4d4d8',
    400: '#ffa1a1aa',
    500: '#ff71717a',
    600: '#ff52525b',
    700: '#ff3f3f46',
    800: '#ff27272a',
    900: '#ff18181b',
  };
  colorsAsMap['coolGray'] = {
    50: '#fff9fafb',
    100: '#fff3f4f6',
    200: '#ffe5e7eb',
    300: '#ffd1d5db',
    400: '#ff9ca3af',
    500: '#ff6b7280',
    600: '#ff4b5563',
    700: '#ff374151',
    800: '#ff1f2937',
    900: '#ff111827',
  };
  colorsAsMap['blueGray'] = {
    50: '#fff8fafc',
    100: '#fff1f5f9',
    200: '#ffe2e8f0',
    300: '#ffcbd5e1',
    400: '#ff94a3b8',
    500: '#ff64748b',
    600: '#ff475569',
    700: '#ff334155',
    800: '#ff1e293b',
    900: '#ff0f172a',
  };
  return colorsAsMap;
}

const Map<String, Color> _SOURCE = {
  "transparent": Colors.transparent,
  "black": Colors.black,
  "white": Colors.white,
  "green": Colors.green,
  "greenAccent": Colors.greenAccent,
  "lightGreen": Colors.lightGreen,
  "lightGreenAccent": Colors.lightGreenAccent,
  "blue": Colors.blue,
  "blueAccent": Colors.blueAccent,
  "blueGrey": Colors.blueGrey,
  "lightBlue": Colors.lightBlue,
  "lightBlueAccent": Colors.lightBlueAccent,
  "red": Colors.red,
  "redAccent": Colors.redAccent,
  "orange": Colors.orange,
  "orangeAccent": Colors.orangeAccent,
  "deepOrange": Colors.deepOrange,
  "deepOrangeAccent": Colors.deepOrangeAccent,
  "brown": Colors.brown,
  "amber": Colors.amber,
  "amberAccent": Colors.amberAccent,
  "yellow": Colors.yellow,
  "yellowAccent": Colors.yellowAccent,
  "cyan": Colors.cyan,
  "cyanAccent": Colors.cyanAccent,
  "purple": Colors.purple,
  "purpleAccent": Colors.purpleAccent,
  "deepPurple": Colors.deepPurple,
  "deepPurpleAccent": Colors.deepPurpleAccent,
  "grey": Colors.grey,
  "indigo": Colors.indigo,
  "indigoAccent": Colors.indigoAccent,
  "lime": Colors.lime,
  "limeAccent": Colors.limeAccent,
  "pink": Colors.pink,
  "pinkAccent": Colors.pinkAccent,
  "teal": Colors.teal,
  "tealAccent": Colors.tealAccent
};

