class PluginConfig {
  static Map<String, Map<String, dynamic>> userConfiguration = {};

  static void configure(Map<String, Map<String, dynamic>> config) {
    userConfiguration = config;
  }

  final SizingConfig $;

  PluginConfig._(this.$);

  factory PluginConfig.withSizingConfig(Map<String, dynamic> json) {
    var sizingConfig = SizingConfig.fromJson(json);

    return PluginConfig._(sizingConfig);
  }

  @override
  String toString() {
    return 'PluginConfig{sizing config: ${$}';
  }

  operator [](String key) {
    switch (key) {
      case "xs":
        return $.xs;
      case "sm":
        return $.sm;
      case "md":
        return $.md;
      case "base":
        return $.base;
      case "default":
        return $.base;
      case "lg":
        return $.lg;
      case "xl":
        return $.xl;
      case "2xl":
        return $.xxl;
      case "3xl":
        return $.xxxl;
      case "4xl":
        return $.xxxxl;
      case "5xl":
        return $.xxxxxl;
      case "6xl":
        return $.xxxxxxl;
      default:
        return null;
    }
  }
}

class SizingConfig {
  final xs;
  final sm;
  final md;
  final base;
  final lg;
  final xl;
  final xxl;
  final xxxl;
  final xxxxl;
  final xxxxxl;
  final xxxxxxl;

  SizingConfig({
    this.xs,
    this.sm,
    this.md,
    this.base,
    this.lg,
    this.xl,
    this.xxl,
    this.xxxl,
    this.xxxxl,
    this.xxxxxl,
    this.xxxxxxl,
  });

  factory SizingConfig.fromJson(Map<String, dynamic> json) {
    var _default = json["default"] ?? json["base"];
    return SizingConfig(
      xs: _parseDouble(json['xs']),
      sm: _parseDouble(json['sm']),
      md: _parseDouble(json['md']),
      base: _parseDouble(_default),
      lg: _parseDouble(json['lg']),
      xl: _parseDouble(json['xl']),
      xxl: _parseDouble(json['2xl']),
      xxxl: _parseDouble(json['3xl']),
      xxxxl: _parseDouble(json['4xl']),
      xxxxxl: _parseDouble(json['5xl']),
      xxxxxxl: _parseDouble(json['6xl']),
    );
  }

  Map<String, dynamic> toJson() => {
        'xs': xs,
        'sm': sm,
        'md': md,
        'base': base,
        'lg': lg,
        'xl': xl,
        '2xl': xxl,
        '3xl': xxxl,
        '4xl': xxxxl,
        '5xl': xxxxxl,
        '6xl': xxxxxxl,
      };

  @override
  String toString() {
    return 'SizingConfig{xs: $xs, sm: $sm, base: $base, lg: $lg, xl: $xl, xxl: $xxl, xxxl: $xxxl}';
  }
}

dynamic _parseDouble(dynamic value) {
  if (value == null) {
    return null;
  }

  if (value is int) {
    return value.toDouble();
  }

  if (value is String) {
    return double.parse(value);
  }

  return value;
}
