import 'package:apply/core/plugin.dart';
import 'package:apply/core/range.dart';
import 'package:apply/core/regex_constants.dart';
import 'package:apply/core/value.dart';

mixin PredefinedSizes<T> on Plugin {
  T get defaultValue;

  T get noneValue;

  String get key;

  T? getValue([String? side]) {
    if (values.hasRange([0])) {
      return defaultValue;
    } else if (values.hasRange([111])) {
      return values[111]!.asDouble() as T;
    }

    String _key = key;
    if (side != null) {
      if (side.contains("|")) {
        _key = "";
        var sides = side.split('|');
        for (var side in sides) {
          _key += "$key-$side|";
        }
        _key = _key.substring(0, _key.length - 1);
      } else {
        _key = "$key-$side";
      }
    }

    T? value;
    Value? valueObject = values[_key];

    if (valueObject == null) {
      return value;
    }

    if ([113].contains(valueObject.range)) {
      return valueObject.asDouble() as T;
    }

    String size = valueObject.asString();
    // if (size != null) {
      value = config[size];
    // }

    return value;
  }

  SidesValue<T> getSideValues() {
    T? defaultValue = getValue();

    if (defaultValue == null) {
      defaultValue = noneValue;
    }

    var top = getValue("tl|tr|t") ?? defaultValue;
    var bottom = getValue("bl|br|b") ?? defaultValue;
    var right = getValue("tr|br|r") ?? defaultValue;
    var left = getValue("tl|bl|l") ?? defaultValue;

    return SidesValue(top: top, bottom: bottom, right: right, left: left);
  }
}

class SizesRange extends Range {
  SizesRange(String key) : super(RegExp(r"^" + "$key-(" + RegexConstants.sizes + r")$"), 101);
}

class SizesRangeDigit extends Range {
  SizesRangeDigit(String key) : super(RegExp(r"^" + "$key" + r"-\d{1,3}$"), 111);
}

class SidesRange extends Range {
  SidesRange(String key)
      : super(RegExp(r"^" + "$key-(" + RegexConstants.tRBL + ")-(" + RegexConstants.sizes + r")$"), 103);
}

class SidesRangeDigit extends Range {
  SidesRangeDigit(String key) : super(RegExp(r"^" + "$key-(" + RegexConstants.tRBL + ")" + r"-\d{1,3}$"), 113);
}

class SidesValue<T> {
  final T? top;
  final T? left;
  final T? bottom;
  final T? right;

  SidesValue({
    this.top,
    this.left,
    this.bottom,
    this.right,
  });

  @override
  String toString() {
    return 'SidesValue{top: $top, left: $left, bottom: $bottom, right: $right}';
  }
}
