import 'package:apply/core/framework.dart';
import 'package:apply/core/range.dart';

class Value {
  final String value;

  //za sada samo prvi karakteri slova vecinom.
  //fleksibilan je sistem, jer za neke ovo moze biti problem a za neke nije.
  final String key;
  final int range;

  Value(this.value, [this.range = 1]) : this.key = _getKeyFromValue(value, range);

  int asInt() => int.parse(value.split("-").last);

  double asDouble() => double.parse(value.split("-").last);

  String asString() => value.split("-").last;

  Pair<String, String> asStringPair() {
    var parts = value.split("-");
    return Pair(parts[parts.length - 2], parts[parts.length - 1]);
  }

  @override
  String toString() {
    return 'Value{value: $value, key: $key, range: $range}';
  }
}

class ValuesList {
  List<Value> values = [];

  void add(Value item) => values.add(item);

  Value? operator [](Object key) {
    Value? value;

    if(key is String){
      var keys = key.split("|");
      for (var key in keys) {
        var filtered = values.where((element) => element.range != 0);
        for (var item in filtered) {
          if (item.key == key) {
            value = item;
          }
        }
      }
    }else if(key is int){
      var filtered = values.where((element) => element.range == key);
      for (var item in filtered) {
          value = item;
      }
    }
    return value;
  }

  @override
  String toString() {
    return 'ValuesList{values: $values}';
  }

  bool has(String key) {
    for (var value in values) {
      if (value.key == key) {
        return true;
      }
    }
    return false;
  }

  bool hasRange(List<int> ranges) {
    for (var value in values) {
      if (ranges.contains(value.range)) {
        return true;
      }
    }
    return false;
  }

  Pair<double, double> getXYPair(
    String key, {
    bool includeXY = true,
    double xDefault = 0,
    double yDefault = 0,
  }) {
    double x = xDefault;
    double y = yDefault;
    if (includeXY) {
      double? value = this[key]?.asDouble();

      if (value != null) {
        x = y = value;
      }

      double? valueM = this["-$key"]?.asDouble();
      if (valueM != null) {
        x = y = -1 * valueM;
      }
    }
    double? xP = this["$key-xy|$key-x"]?.asDouble() ?? null;
    double? xM = this["-$key-xy|-$key-x"]?.asDouble() ?? null;
    double? yP = this["$key-xy|$key-y"]?.asDouble() ?? null;
    double? yM = this["-$key-xy|-$key-y"]?.asDouble() ?? null;

    if (xM != null) {
      xM = xM * -1;
    }
    if (yM != null) {
      yM = yM * -1;
    }
    return Pair(xM ?? xP ?? x, yM ?? yP ?? y);
  }
}

String _getKeyFromValue(String value, int range) {
  var parts = value.split("-");
  int _value = range % 10;
  String key;

  switch (_value) {
    case -1:
      key = value;
      break;
    case 1:
      key = parts.first;
      break;
    case 2:
      key = parts.first;
      break;
    case 3:
      key = parts[0] + "-" + parts[1];
      break;
    case 4:
      key = parts[0] + "-" + parts[1] + "-" + parts[2];
      break;
    default:
      key = parts.first;
      break;
  }

  if (key.isEmpty) {
    key = "-${parts[1]}";
  }
  return key;
}
