import 'package:apply/colors/apply_colors.dart';
import 'package:flutter/cupertino.dart';
import 'package:apply/core/framework.dart';
import 'package:apply/core/plugin.dart';
import 'package:apply/core/range.dart';
import 'package:apply/core/regex_constants.dart';
import 'package:apply/core/value.dart';
import 'package:apply/elements/div.dart';
import 'package:apply/elements/element_class.dart';
import 'package:apply/plugins/background_opacity.dart';
/*
* [A-Za-z]+
*
* Range(RegExp("^bg-(${RegexConstants.colors})\$"), 1),
    Range(RegExp("^bg-(${RegexConstants.colors})-(${RegexConstants.shade})\$"), 11),
* */
class BackgroundColor extends Plugin {
  final List<Range> ranges = [
    Range(RegExp("^bg-((0x){0,1}|#{0,1})([0-9A-F]{8}|[0-9A-F]{6})\$"), 11),
    Range(RegExp("^bg-(?!opacity|opacity)[A-Za-z]+\$"), 1),
    Range(RegExp("^bg-(?!opacity|opacity)[A-Za-z]+-(${RegexConstants.shade})\$"), 2),
  ];
  final String id = "backgroundColor";

  BackgroundColor([String? value]) {
    if (value != null) {
      addValue(value);
    }
  }

  @override
  BackgroundColor createInstance(ElementClass elementClass) => BackgroundColor(elementClass.value);

  Color? getColor(Div element) {
    BackgroundOpacity? backgroundOpacity = element.plugins?.getPluginById("backgroundOpacity") as BackgroundOpacity?;
    Color? color;
    Value? colorValue = values["bg"];
    if(colorValue != null){
      var colorAsString = colorValue.asString();
      if(colorAsString[0] == "#"){
        // color = HexColor.fromHex(colorAsString);
        color = ApplyColors.fromHex(colorAsString);
      }else{
        if(colorValue.range == 1){
          color = ApplyColors.getColor(colorAsString + "-500");
        }else if(colorValue.range == 2){
          var pair = colorValue.asStringPair();
          color = ApplyColors.getColor("${pair.first}-${pair.last}");
        }
      }
    }

    if (backgroundOpacity != null && backgroundOpacity.getColorOpacity() < 1) {
      // print(["Opacity je: ", backgroundOpacity.getColorOpacity()]);
      var colorOpacity = backgroundOpacity.getColorOpacity();
      color = color?.withOpacity(colorOpacity);
    }

    return color;
  }

}


extension HexColor on Color {
  /// String is in the format "aabbcc" or "ffaabbcc" with an optional leading "#".
  static Color fromHex(String hexString) {
    final buffer = StringBuffer();
    if (hexString.length == 6 || hexString.length == 7) buffer.write('ff');
    buffer.write(hexString.replaceFirst('#', ''));
    return Color(int.parse(buffer.toString(), radix: 16));
  }

  /// Prefixes a hash sign if [leadingHashSign] is set to `true` (default is `true`).
  String toHex({bool leadingHashSign = true}) => '${leadingHashSign ? '#' : ''}'
      '${alpha.toRadixString(16).padLeft(2, '0')}'
      '${red.toRadixString(16).padLeft(2, '0')}'
      '${green.toRadixString(16).padLeft(2, '0')}'
      '${blue.toRadixString(16).padLeft(2, '0')}';
}
