import 'package:flutter/cupertino.dart';
import 'package:apply/core/framework.dart';
import 'package:apply/core/plugin.dart';
import 'package:apply/core/range.dart';
import 'package:apply/core/regex_constants.dart';
import 'package:apply/elements/div.dart';
import 'package:apply/elements/element_class.dart';
import 'package:apply/plugins/background_size.dart';

class BackgroundImage extends Plugin {
  final List<Range> ranges = [
    Range(RegExp(r"^bg-none$"), 0),
    Range(RegExp("^bg-gradient-to-(${RegexConstants.tRBL})\$"), 1),
    Range(RegExp(r"^bg-asset-image-.*\.(gif|jpe?g|bmp|png)$"), 2),
  ];
  final String value;
  final String id = "backgroundImage";

  BackgroundImage([this.value = "bg-none"]) {
    addValue(value);
  }

  @override
  BackgroundImage createInstance(ElementClass elementClass) {
    return BackgroundImage(elementClass.value);
  }

  BackgroundImageMode get modeOfOperation {
    var range = values.values.last.range;

    if(range == 1){
      return BackgroundImageMode.gradient;
    }else if(range == 2){
      return BackgroundImageMode.assetImage;
    }

    return BackgroundImageMode.none;
  }

  Pair<Alignment, Alignment>? getBeginEndPoints() {
    var value = values.values.last.asString();
    Pair<Alignment, Alignment>? pair;
    switch (value) {
      case "none":
        pair = null;
        break;
      case "r":
        pair = Pair(Alignment.centerLeft, Alignment.centerRight);
        break;
      case "l":
        pair = Pair(Alignment.centerRight, Alignment.centerLeft);
        break;
      case "t":
        pair = Pair(Alignment.bottomCenter, Alignment.topCenter);
        break;
      case "b":
        pair = Pair(Alignment.topCenter, Alignment.bottomCenter);
        break;
      case "br":
        pair = Pair(Alignment.topLeft, Alignment.bottomRight);
        break;
      case "tr":
        pair = Pair(Alignment.bottomLeft, Alignment.topRight);
        break;
      case "bl":
        pair = Pair(Alignment.topRight, Alignment.bottomLeft);
        break;
      case "tl":
        pair = Pair(Alignment.bottomRight, Alignment.topLeft);
        break;
      default:
        pair = Pair(Alignment.topCenter, Alignment.bottomCenter);
    }
    return pair;
  }

  String getImageValue()=> values.values.first.asString();

  DecorationImage getAssetImageDecoration(Div element) {
    var backgroundSize = (element.plugins?.getPluginByIdOrDefault("backgroundSize") as BackgroundSize).getBackgroundSize();
    return DecorationImage(
      image: AssetImage(getImageValue()),
      fit: backgroundSize,
    );
  }
}


enum BackgroundImageMode{
  gradient, assetImage, none,
}
