import 'package:flutter/material.dart';
import 'package:apply/core/config/plugin_config.dart';
import 'package:apply/core/plugin.dart';
import 'package:apply/core/predefined.sizes.dart';
import 'package:apply/core/range.dart';
import 'package:apply/elements/element_class.dart';

class BorderRadiusP extends Plugin with PredefinedSizes<double> {
  final List<Range> ranges = [
    Range(RegExp(r"^rounded$"), 0),
    Range(RegExp(r"^rounded-full$"), 1),
    SizesRange("rounded"),
    SidesRange("rounded"),
    SizesRangeDigit("rounded"),
    SidesRangeDigit("rounded"),
  ];
  final String id = "borderRadius";
  final String value;

  BorderRadiusP([this.value = "rounded"]) {
    setConfig();
    addValue(value);
  }

  @override
  BorderRadiusP createInstance(ElementClass elementClass) {
    return BorderRadiusP(elementClass.value);
  }

  BorderRadius getBorderRadius() {
    double? radius = getValue();

    if (values.hasRange([1])) {
      radius = 9999;
    }

    double? topLeft = getValue("tl");
    double? topRight = getValue("tr");
    double? bottomLeft = getValue("bl");
    double? bottomRight = getValue("br");

    if (topLeft != null || topRight != null || bottomLeft != null || bottomRight != null) {
      return BorderRadius.only(
        topLeft: Radius.circular(topLeft ?? 0),
        topRight: Radius.circular(topRight ?? 0),
        bottomLeft: Radius.circular(bottomLeft ?? 0),
        bottomRight: Radius.circular(bottomRight ?? 0),
      );
    } else {
      return BorderRadius.all(Radius.circular(radius!));
    }
  }

  @override
  Map<String, dynamic> defaultConfig = {
    'sizing': SizingConfig(
      xs: 1.25,
      sm: 2.5,
      base: 5,
      md: 7.5,
      lg: 10,
      xl: 15,
      xxl: 20,
      xxxl: 30,
    ).toJson(),
  };

  @override
  double get defaultValue => config["default"];

  @override
  double get noneValue => 0;

  @override
  String get key => "rounded";
}
