import 'package:flutter/material.dart';
import 'package:apply/core/config/plugin_config.dart';
import 'package:apply/core/framework.dart';
import 'package:apply/core/plugin.dart';
import 'package:apply/core/range.dart';
import 'package:apply/core/regex_constants.dart';
import 'package:apply/elements/element_class.dart';
import 'package:apply/elements/div.dart';
import 'package:apply/plugins/border_color.dart';


class BorderWidth extends Plugin{
  final List<Range> ranges = [
    Range(RegExp(r"^border$"), 1),
    Range(RegExp(r"^border-\d{1,3}$"), 2),
    Range(RegExp(r"^border-(" + RegexConstants.tRBL + r")-\d{1,3}$"), 3),
    Range(RegExp(r"^border-none$"), -2),
  ];
  final String id = "borderWidth";
  final String value;

  BorderWidth([this.value = "border"]) {
    setConfig();
    if (value != null) {
      addValue(value);
    }
  }

  double getWidth() {
    double defaultValue = config.$.base;
    double borderWidth = defaultValue;
    if (values.values.length > 0 && values.values.last.range == 2) {
      borderWidth = values["border"]?.asDouble() ?? defaultValue;
    }
    return borderWidth;
  }

  Border? getBorder(Div element) {
    if(values.hasRange([-2])){
      return null;
    }
    var plugins = element.plugins;
    BorderColor borderColor = plugins?.getPluginByIdOrDefault("borderColor") as BorderColor;
    var color = borderColor.getColor();
    var width = getWidth();

    double? top = values["border-tr|border-tl|border-t"]?.asDouble() ?? null;
    double? right = values["border-tr|border-br|border-r"]?.asDouble() ?? null;
    double? bottom = values["border-br|border-bl|border-b"]?.asDouble() ?? null;
    double? left = values["border-tl|border-bl|border-l"]?.asDouble() ?? null;

    if (top != null || right != null || bottom != null || left != null) {
      return Border(
        top: BorderSide(width: top ?? 0, color: color),
        right: BorderSide(width: right ?? 0, color: color),
        bottom: BorderSide(width: bottom ?? 0, color: color),
        left: BorderSide(width: left ?? 0, color: color),
      );
    } else {
      return Border.all(width: width, color: color);
    }
  }

  @override
  BorderWidth createInstance(ElementClass elementClass) => BorderWidth(elementClass.value);

  @override
  Map<String, dynamic> defaultConfig = {
    'sizing': SizingConfig(
      xs: 1,
      base: 1,
    ).toJson(),
  };


}
