import 'package:apply/apply.dart';
import 'package:apply/core/plugin.dart';
import 'package:apply/core/predefined.sizes.dart';
import 'package:apply/core/range.dart';
import 'package:apply/core/regex_constants.dart';
import 'package:apply/core/value.dart';
import 'package:apply/elements/element_class.dart';
import 'package:flutter/material.dart' as m;

class BoxShadowP extends Plugin with PredefinedSizes<_ShadowValues> {
  final List<Range> ranges = [
    SizesRange("shadow"),
    SidesRange("shadow"),
    Range(RegExp("^shadow-color-[A-Za-z]+\$"), 3),
    Range(RegExp(r"^shadow$"), -1),
    Range(RegExp(r"^shadow-none$"), -2),
    Range(RegExp("^shadow-color-[A-Za-z]+-(${RegexConstants.shade})\$"), 13),
  ];
  final String id = "boxShadow";

  BoxShadowP([String? value]) {
    setConfig();
    if (value != null) {
      addValue(value);
    }
  }

  @override
  BoxShadowP createInstance(ElementClass elementClass) {
    return BoxShadowP(elementClass.value);
  }

  @override
  _ShadowValues get defaultValue => _ShadowValues(0, 1, 3, 0);

  @override
  _ShadowValues get noneValue => _ShadowValues(0, 0, 0, 0);

  @override
  String get key => "shadow";

  Map<String, dynamic> defaultConfig = {
    'sizing': {
      "md": _ShadowValues(0, 4, 6, -1),
      "lg": _ShadowValues(0, 10, 15, -3),
      "xl": _ShadowValues(0, 20, 25, -5),
      "xxl": _ShadowValues(0, 25, 50, -12),
    }
  };

//
  m.BoxShadow? getShadow() {
    if (values.hasRange([-2])) {
      return null;
    }
    _ShadowValues _shadowValues = defaultValue;
    String? sizeValue = values["shadow"]?.asString();
    if (sizeValue != null) {
      switch (sizeValue) {
        case "md":
          _shadowValues = _ShadowValues(0, 4, 6, -1);
          break;
        case "lg":
          _shadowValues = _ShadowValues(0, 10, 15, -3);
          break;
        case "xl":
          _shadowValues = _ShadowValues(0, 20, 25, -5);
          break;
        case "xxl":
          _shadowValues = _ShadowValues(0, 25, 50, -12);
          break;
      }
    }

    m.Color color = m.Colors.grey.withOpacity(0.5);

    Value? valueFrom = values["shadow-color"];
    if (valueFrom != null) {
      if (valueFrom.range == 3) {
        color = ApplyColors.getColor(valueFrom.asString() + "-500");
      } else if (valueFrom.range == 13) {
        var pair = valueFrom.asStringPair();
        color = ApplyColors.getColor("${pair.first}-${pair.last}");
      }
    }

    return m.BoxShadow(
      color: color,
      offset: m.Offset(
        _shadowValues.offsetX,
        _shadowValues.offsetY,
      ),
      blurRadius: _shadowValues.blurRadius,
      spreadRadius: _shadowValues.spreadRadius,
    );
  }
}

class _ShadowValues {
  final double offsetX;
  final double offsetY;
  final double blurRadius;
  final double spreadRadius;

  _ShadowValues(this.offsetX, this.offsetY, this.blurRadius, this.spreadRadius);

  @override
  String toString() {
    return '_ShadowValues{offsetY: $offsetY}';
  }
}
