import 'package:flutter/material.dart';
import 'package:apply/core/framework.dart';
import 'package:apply/core/plugin.dart';
import 'package:apply/core/range.dart';
import 'package:apply/elements/div.dart';
import 'package:apply/elements/element_class.dart';
import 'package:apply/plugins/flex_grow.dart';

class FlexDirection extends Plugin {
  final List<Range> ranges = [
    Range(new RegExp(r"^flex-(row|col)$"), 1),
  ];

  //Ovdje treba default jer se ovaj plugin tako i koristi, bilo da ima ili da nema, uvijek se uzima direction.
  FlexDirection([String value = "flex-row"]) {
    setConfig();
    addValue(value);
  }

  final String id = "flexDirection";

  String get getValue => values["flex"]!.asString();

  @override
  FlexDirection createInstance(ElementClass elementClass) => FlexDirection(elementClass.value);

  Widget? handle(Div element, BuildContext context) {
    var children = element.children;
    _processChildrenAsFlexItems(children, context);

    if (getValue == "row") {
      return Row(
        mainAxisSize: MainAxisSize.min,
      );
    } else if (getValue == "col") {
      return Column(
        mainAxisSize: MainAxisSize.min,
      );
    }
  }

  void _processChildrenAsFlexItems(List<Widget> children, BuildContext context) {
    for (var i = 0; i < children.length; i++) {
      var child = children[i];
      if (child.hasDiv(context)) {
        FlexGrow? flexGrowPlugin = child.setOwnPlugins(context)?.ownPlugins?.getPluginById("flexGrow") as FlexGrow?;
        if (flexGrowPlugin != null) {
          // print("value je ${flexGrowPlugin.getValue}");
          var flexFactor = flexGrowPlugin.getValue;
          children[i] = Expanded(
            child: child,
            flex: flexFactor,
          );
        }
      }
    }
  }
}
