import 'package:flutter/cupertino.dart';
import 'package:apply/colors/apply_colors.dart';
import 'package:apply/core/framework.dart';
import 'package:apply/core/plugin.dart';
import 'package:apply/core/range.dart';
import 'package:apply/core/regex_constants.dart';
import 'package:apply/core/value.dart';
import 'package:apply/elements/element_class.dart';

class GradientColorStops extends Plugin {
  final List<Range> ranges = [
    Range(RegExp("^from-(${RegexConstants.colors})\$"), 1),
    Range(RegExp("^via-(${RegexConstants.colors})\$"), 1),
    Range(RegExp("^to-(${RegexConstants.colors})\$"), 1),
    Range(RegExp("^from-(${RegexConstants.colors})-(${RegexConstants.shade})\$"), 2),
    Range(RegExp("^via-(${RegexConstants.colors})-(${RegexConstants.shade})\$"), 2),
    Range(RegExp("^to-(${RegexConstants.colors})-(${RegexConstants.shade})\$"), 2),
  ];
  final String? value;
  final String id = "gradientColorStops";

  GradientColorStops([this.value]){
    if(value != null){
    addValue(value);
    }
  }

  @override
  GradientColorStops createInstance(ElementClass elementClass) {
    return GradientColorStops(elementClass.value);
  }

  List<Color?> getColors() {
     Color? from;
     Color? via;
     Color? to;
    Value? valueFrom = values["from"];
    if(valueFrom != null){
      if(valueFrom.range == 1){
        from = ApplyColors.getColor(valueFrom.asString() + "-500");
      }else if(valueFrom.range == 2){
        var pair = valueFrom.asStringPair();
        from = ApplyColors.getColor("${pair.first}-${pair.last}");
      }
    }

    Value? valueVia = values["via"];
    if(valueVia != null){
      if(valueVia.range == 1){
        via = ApplyColors.getColor(valueVia.asString() + "-500");
      }else if(valueVia.range == 2){
        var pair = valueVia.asStringPair();
        via = ApplyColors.getColor("${pair.first}-${pair.last}");
      }
    }

    Value? valueTo = values["to"];
    if(valueTo != null){
      if(valueTo.range == 1){
        to = ApplyColors.getColor(valueTo.asString() + "-500");
      }else if(valueTo.range == 2){
        var pair = valueTo.asStringPair();
        to = ApplyColors.getColor("${pair.first}-${pair.last}");
      }
    }

    return [from, via, to];
  }

  LinearGradient? getGradient(Pair<Alignment, Alignment> beginEndPoints){
    var colors = getColors();
    if(colors[0] != null && colors[2] != null ){
      var applyColors = colors.where((element) => element != null).toList(growable: false).cast<Color>();
      return LinearGradient(
        begin: beginEndPoints.first!,
        end: beginEndPoints.last!,
        colors: applyColors, // red to yellow
        tileMode: TileMode.repeated, // repeats the gradient over the canvas
      );
    }
    return null;
  }
}
