import 'package:flutter/material.dart';
import 'package:flutter_staggered_grid_view/flutter_staggered_grid_view.dart';
import 'package:apply/core/framework.dart';
import 'package:apply/core/plugin.dart';
import 'package:apply/core/range.dart';
import 'package:apply/elements/div.dart';
import 'package:apply/elements/element_class.dart';

class GridP extends Plugin {
  final List<Range> ranges = [
    Range(new RegExp(r"(grid-|gap-)"), 0),
    Range(new RegExp(r"^(row|col)-span-\d{1,3}$"), 3),
  ];
  final List<Range> cRanges = [
    Range(new RegExp(r"^grid-flow-(row|col)$"), 13),
    Range(new RegExp(r"^grid-cols-\d{1,3}$"), 23),
    Range(new RegExp(r"^grid-rows-\d{1,3}$"), 33),
    Range(new RegExp(r"^gap-\d{1,3}$"), 71),
    Range(new RegExp(r"^gap-x-\d{1,3}$"), 73),
    Range(new RegExp(r"^gap-y-\d{1,3}$"), 73),
  ];
  var scaleRanges = [12, 13, 23];
  var translateRanges = [21, 23, 24, 33, 34];
  var skewRanges = [51, 53, 54, 63, 64];
  var originRanges = [71, 73, 74, 83, 84];
  var rotationRanges = [42, 43];

  final String? value;
  final String id = "grid";

  GridP([this.value]) {
    setConfig();
    if (value != null) {
      addValue(value);
    }
  }

  @override
  GridP createInstance(ElementClass elementClass) => GridP(elementClass.value);

  Widget handle(Div element, BuildContext context) {
    String gridFlow = "row";

    if(values.hasRange([13])){
      gridFlow = values["grid-flow"]!.asString();
    }
    if(values.hasRange([23])){
      gridFlow = "col";
    }
    if(values.hasRange([33])){
      gridFlow = "row";
    }
    /*
    *
    *   Range(new RegExp(r"^grid-flow-(row|col)$"), 13),
    Range(new RegExp(r"^grid-cols-\d{1,3}$"), 23),
    Range(new RegExp(r"^grid-rows-\d{1,3}$"), 33),
    *
    * */
    var rowsColsDefaultCount = 1;
    int rowsCount = values["grid-rows"]?.asInt() ?? rowsColsDefaultCount;
    int colsCount = values["grid-cols"]?.asInt() ?? rowsColsDefaultCount;
    int crossAxisCount;
    if (gridFlow == "col") {
      crossAxisCount = colsCount;
    } else {
      crossAxisCount = rowsCount;
    }
    var tiles = <StaggeredTile>[];
    var xyGap = getXYGap();
    double mainAxisSpacing;
    double crossAxisSpacing;
    if (gridFlow == "col") {
      mainAxisSpacing = xyGap.last ?? 0;
      crossAxisSpacing = xyGap.first ?? 0;
    } else {
      mainAxisSpacing = xyGap.first ?? 0;
      crossAxisSpacing = xyGap.last ?? 0;
    }
    Axis scrollDirection = gridFlow == "col" ? Axis.vertical : Axis.horizontal;
    var children = element.children;
    var spanInfo = _processChildrenAsGridItems(children, context);
    var itemsCount = children.length;

    // print(spanInfo);
    for (var span in spanInfo) {
      int crossAxisCellCount;
      double? mainAxisCellCount;
      double? size;
      if (gridFlow == "col") {
        crossAxisCellCount = span.last ?? 1;
        mainAxisCellCount = (span.first ?? 1).toDouble();
        size = span.widthAndHeightPair?.last;
        if(size != null && size > 0 && size < 1){
          size = MediaQuery.of(context).size.width * size;
        }
      } else {
        crossAxisCellCount = span.first ?? 1;
        mainAxisCellCount = (span.last ?? 1).toDouble();
        size = span.widthAndHeightPair?.first;
      }
      // tiles.add(StaggeredTile.extent(crossAxisCellCount, 200));
      // tiles.add(StaggeredTile.fit(crossAxisCellCount));

      if (size == null) {
        tiles.add(StaggeredTile.count(crossAxisCellCount, mainAxisCellCount));
      }else{
        tiles.add(StaggeredTile.extent(crossAxisCellCount, size));
      }


    }

    return StaggeredGridView.count(
      scrollDirection: scrollDirection,
      crossAxisCount: crossAxisCount,
      staggeredTiles: tiles,
      children: children,
      mainAxisSpacing: mainAxisSpacing,
      crossAxisSpacing: crossAxisSpacing,
      addAutomaticKeepAlives: true,
      shrinkWrap: true,
      padding: EdgeInsets.only(bottom: 0),
      physics: NeverScrollableScrollPhysics(),
    );
  }

  Pair<int, int> getRowColSpan() {
    int? rowSpan = values["row-span"]?.asInt();
    int? colSpan = values["col-span"]?.asInt();

    return Pair(rowSpan, colSpan);
  }

  Pair<double, double> getXYGap() {
    var $ = values.getXYPair("gap");
    return $;
  }

  List<TileInfo> _processChildrenAsGridItems(List<Widget> children, BuildContext context) {
    var itemsInfo = <TileInfo>[];
    for (var i = 0; i < children.length; i++) {
      Pair<int,int>? rowColSpan;
      TileInfo? info;
      var child = children[i];

      if (child.hasDiv(context)) {
        var div = child.setOwnPlugins(context);
        GridP? gridPlugin = div?.ownPlugins?.getPluginById("grid") as GridP?;
        if (gridPlugin != null) {
          rowColSpan = gridPlugin.getRowColSpan();

          // print(["Info", div.widthAndHeightPair, div.ownPlugins]);

        }

          info = TileInfo(rowColSpan?.first!, rowColSpan?.last!, div?.widthAndHeightPair);
      }
      itemsInfo.insert(i, info!);
    }
    return itemsInfo;
  }
}

class TileInfo {
  final int? first;
  final int? last;
  final Pair<double, double>? widthAndHeightPair;

  TileInfo(
    this.first,
    this.last, [
    this.widthAndHeightPair,
  ]);
}
