import 'package:flutter/material.dart';
import 'package:apply/core/plugin.dart';
import 'package:apply/core/range.dart';
import 'package:apply/elements/div.dart';
import 'package:apply/elements/element_class.dart';
import 'package:apply/core/framework.dart';

class Space extends Plugin {
  final String id = "space";
  final List<Range> ranges = [
    Range(new RegExp(r"^space-\d{1,3}$"), 1),
    Range(new RegExp(r"^space-x-\d{1,3}$"), 3),
    Range(new RegExp(r"^space-y-\d{1,3}$"), 13),
  ];

  Space([String? value]) {
    setConfig();
    addValue(value);
  }

  void handleFlex(Div element, BuildContext context, Axis direction) {
    _addSpacing(element, context, direction);
  }

  void _addSpacing(Div element, BuildContext context, Axis direction) {
    double? width = values["space|space-x"]?.asDouble() ?? null;
    double? height = values["space|space-y"]?.asDouble() ?? null;
    List<Widget> result = [];
    var children = element.children;
    var length = children.length;
    for (var i = 0; i < length; i++) {
      result.add(children[i]);
      if(i < (length - 1)){
        result.add(SizedBox(height: height, width: width,));
      }
    }

    element.children.replaceRange(0, element.children.length, result);

  }

  Pair<double,double> xYPair() => Pair(values["space|space-x"]?.asDouble() ?? 0,values["space|space-y"]?.asDouble() ?? 0);

  @override
  Space createInstance(ElementClass elementClass) => Space(elementClass.value);
}
