import 'package:flutter/cupertino.dart';
import 'package:apply/colors/apply_colors.dart';
import 'package:apply/core/framework.dart';
import 'package:apply/core/plugin.dart';
import 'package:apply/core/range.dart';
import 'package:apply/core/regex_constants.dart';
import 'package:apply/core/value.dart';
import 'package:apply/elements/div.dart';
import 'package:apply/elements/element_class.dart';
import 'package:apply/plugins/background_opacity.dart';
import 'package:apply/plugins/text_opacity.dart';

class TextColorP extends Plugin {
  final List<Range> ranges = [
    Range(RegExp("^text-(?!opacity|opacity|underline|line-through|overline|none|no-underline)[A-Za-z]+\$"), 1),
    Range(RegExp("^text-(?!opacity|opacity|underline|line-through|overline|none|no-underline)[A-Za-z]+-(${RegexConstants.shade})\$"), 2),
  ];
  final String? value;
  final String id = "textColor";

  TextColorP([this.value]) {
    if (value != null) {
      addValue(value);
    }
  }

  @override
  TextColorP createInstance(ElementClass elementClass) => TextColorP(elementClass.value);

  Color? getColor(Div element) {
    TextOpacity? textOpacity = element.plugins?.getPluginById("textOpacity") as TextOpacity?;
    Color? color;
    Value? valueFrom = values["text"];
    if(valueFrom != null){
      if(valueFrom.range == 1){
        color = ApplyColors.getColor(valueFrom.asString() + "-500");
      }else if(valueFrom.range == 2){
        var pair = valueFrom.asStringPair();
        color = ApplyColors.getColor("${pair.first}-${pair.last}");
      }
    }

    if (textOpacity != null && textOpacity.getColorOpacity() < 1) {
      var colorOpacity = textOpacity.getColorOpacity();
      color = color?.withOpacity(colorOpacity);
    }

    return color;
  }

}
