import 'package:flutter/material.dart';
import 'package:apply/core/matrix4_transform.dart';
import 'package:apply/core/plugin.dart';
import 'package:apply/core/range.dart';
import 'package:apply/elements/div.dart';
import 'package:apply/elements/element_class.dart';

class TransformP extends Plugin {
  final List<Range> ranges = [
    Range(new RegExp(r"(transform|rotate|translate|skew|scale|origin)"), 0),
  ];
  final List<Range> cRanges = [
    Range(new RegExp(r"^scale-\d{1,3}$"), 12),
    Range(new RegExp(r"^scale-x-\d{1,3}$"), 13),
    Range(new RegExp(r"^scale-y-\d{1,3}$"), 23),
    Range(new RegExp(r"^translate-x-\d{1,3}$"), 23),
    Range(new RegExp(r"^translate-\d{1,3}$"), 21),
    Range(new RegExp(r"^-translate-x-\d{1,3}$"), 24),
    Range(new RegExp(r"^translate-y-\d{1,3}$"), 33),
    Range(new RegExp(r"^-translate-y-\d{1,3}$"), 34),
    Range(new RegExp(r"^rotate-\d{1,3}$"), 42),
    Range(new RegExp(r"^-rotate-\d{1,3}$"), 43),
    Range(new RegExp(r"^skew-\d{1,3}$"), 51),
    Range(new RegExp(r"^skew-x-\d{1,3}$"), 53),
    Range(new RegExp(r"^-skew-x-\d{1,3}$"), 54),
    Range(new RegExp(r"^skew-y-\d{1,3}$"), 63),
    Range(new RegExp(r"^-skew-y-\d{1,3}$"), 64),

    Range(new RegExp(r"^origin-\d{1,3}$"), 71),
    Range(new RegExp(r"^origin-x-\d{1,3}$"), 73),
    Range(new RegExp(r"^-origin-x-\d{1,3}$"), 74),
    Range(new RegExp(r"^origin-y-\d{1,3}$"), 83),
    Range(new RegExp(r"^-origin-y-\d{1,3}$"), 84),
  ];
  var scaleRanges = [12, 13, 23];
  var translateRanges = [21, 23, 24, 33, 34];
  var skewRanges = [51, 53, 54, 63, 64];
  var originRanges = [71, 73, 74, 83, 84];
  var rotationRanges = [42, 43];

  final String? value;
  final String id = "transform";

  TransformP([this.value]) {
    setConfig();
    if (value != null) {
      addValue(value);
    }
  }

  @override
  TransformP createInstance(ElementClass elementClass) => TransformP(elementClass.value);

  dynamic debugggg() {
    // print([
    //   values.getXYPair("scale"),
    //   values.getXYPair("translate"),
    //   values.getXYPair("rotate"),
    //   values.getXYPair("skew"),
    //   values.getXYPair("origin"),
    // ]);
  }

  Widget getTransformedWidget(Div element, WidgetBuilder wb) {
    Matrix4 m;
    Matrix4Transform matrix;

    if (values.hasRange(skewRanges)) {
      var $ = values.getXYPair("skew");
      m = Matrix4.skew(
        Matrix4Transform.toRadians($.first!),
        Matrix4Transform.toRadians($.last!),
      );
    } else {
      m = Matrix4.identity();
    }

    matrix = Matrix4Transform.from(m);

    if (values.hasRange(rotationRanges)) {
      var $ = values.getXYPair("rotate");
      matrix = matrix.rotateDegrees($.first!);
    }

    if (values.hasRange(translateRanges)) {
      var $ = values.getXYPair("translate");
      matrix = matrix.translate(x: $.first!, y: $.last!);
    }

    if (values.hasRange(scaleRanges)) {
      var $ = values.getXYPair("scale", xDefault: 100, yDefault: 100);
      matrix = matrix.scaleBy(x: $.first! / 100, y: $.last! / 100);
    }

    if (values.hasRange(originRanges)) {
      var $ = values.getXYPair("origin");
      matrix = matrix.translate(x: $.first!, y: $.last!);
    }

    return Builder(
      builder: (c) => Transform(
        transform: matrix.matrix4,
        child: wb(c),
      ),
    );
    ;
  }
}
