import 'package:flutter/material.dart';
import 'package:responsive_builder/responsive_builder.dart';

class ResponsiveService {
  static ResponsiveService instance = ResponsiveService._();

  ResponsiveService._() {
    var sm = BreakPoint("sm", 550);
    var md = BreakPoint("md", 800);
    var lg = BreakPoint("lg", 1024);
    ResponsiveSizingConfig.instance
        .setCustomBreakpoints(ScreenBreakpoints(desktop: lg.points, tablet: md.points, watch: sm.points));
  }

  String? getBreakPoint(Size size) {
    var sm = BreakPoint("sm", 550);
    var md = BreakPoint("md", 800);
    var lg = BreakPoint("lg", 1024);
    if(size.width > lg.points){
      return "lg";
    }
    if(size.width > md.points){
      return "md";
    }
    if(size.width > sm.points){
      return "sm";
    }
    return null;
    // var deviceType = getDeviceType(size);
    // switch (deviceType) {
    //   case DeviceScreenType.watch:
    //     return null;
    //   case DeviceScreenType.tablet:
    //     return "md";
    //   case DeviceScreenType.mobile:
    //     return "sm";
    //   case DeviceScreenType.desktop:
    //     return "lg";
    //   default:
    //     return null;
    // }
  }

  List<String> filterBreakPoints(List<String> attributes, Size size) {
    String? breakPoint = getBreakPoint(size);
    var result = <String>[];
    for (var i = 0; i < attributes.length; i++) {
      var attribute = attributes[i];
      var regExp = new RegExp(r"^..:");
      var allStringMatches = _allStringMatches(attribute, regExp);

      if (allStringMatches.length > 0) {
        var key = allStringMatches.first;
        // print(key);
        var dataToAdd = attribute.replaceFirst(key, "");
        if (breakPoint == null) {
        } else if (breakPoint == "sm") {
          result.add(dataToAdd);
        } else if (breakPoint == "md") {
          if (["sm:", "md:"].contains(key)) {
            result.add(dataToAdd);
          }
        } else if (breakPoint == "lg") {
          if (["sm:", "md:", "lg:"].contains(key)) {
            result.add(dataToAdd);
          }
        }
      } else {
        result.add(attribute);
      }
    }
    return result;
  }

  Iterable<String> _allStringMatches(String text, RegExp regExp) => regExp.allMatches(text).map((m) => m.group(0)!);
}

class BreakPoint {
  final String prefix;
  final double points;

  BreakPoint(this.prefix, this.points);
}
